#!/usr/bin/env python

import os
import sys
from optparse import OptionParser
import re

import string
import pylab 

class Table:

    def __init__(self, fn, min_mapq):
        ncol = -1

        # read in the data
        data = list()
        fh = open(fn, 'r')
        for line in fh:
            line = line.rstrip("\r\n")
            if '#' == line[0]:
                continue
            else:
                table = self.__parse(line)
                if table[0] < min_mapq:
                    continue
                data.append(table)
                if -1 == ncol:
                    ncol = len(table)
                elif ncol != len(table):
                    sys.stderr.write("Error: table was misformatted\n")
                    sys.exit(1)
                    
        fh.close()

        # re-arrange the data
        self.matrix = [[data[i][j] for j in xrange(ncol)] for i in xrange(len(data))]

    # NB: assumes sorted descending
    def __parse(self, line):
        tokens = re.findall(r'[\w+-.]+', line)
        for i in range(0, 13):
            tokens[i] = int(tokens[i])
        for i in range(13, len(tokens)):
            tokens[i] = float(tokens[i])
        return tokens

def main(options):
    fmts = ['-', '--', '-.', ':', '.']

    if None == options.fns or 0 == len(options.fns):
        return

    # read in through the files
    fmts_i = 0
    for fn in options.fns:
        table = Table(fn, options.min_mapq)
        # extract sensitivity 
        x = [table.matrix[i][17] for i in range(len(table.matrix))]
        # extract specificity
        y = [table.matrix[i][16] for i in range(len(table.matrix))]
        pylab.plot(x, y, fmts[fmts_i])
        fmts_i = fmts_i + 1
        if len(fmts) <= fmts_i:
            fmts_i = 0
    # plot values
    pylab.title('DWGSIM ROC')
    pylab.xlabel('specificity')
    pylab.ylabel('sensitivity')
    # xlim
    if None != options.xlim:
        nums = re.findall(r'\d+\.?\d*', options.xlim)
        if 2 == len(nums):
            pylab.xlim(xmin=float(nums[0]), xmax=float(nums[1]))
    # ylim
    if None != options.ylim:
        nums = re.findall(r'\d+\.?\d*', options.ylim)
        if 2 == len(nums):
            pylab.ylim(ymin=float(nums[0]), ymax=float(nums[1]))
    # legend
    if not options.no_legend:
        if None != options.ids and len(options.fns) == len(options.ids):
            pylab.legend(options.ids, title='IDs', loc='lower left')
        elif options.infer_ids:
            ids = list()
            for fn in options.fns:
                i = re.sub(r'^.*\/', '', fn)
                i = re.sub(r'\.sam\.txt$', '', i)
                i = re.sub(r'^out', '', i)
                i = re.sub(r'^dwgsim_eval', '', i)
                i = re.sub(r'^\.', '', i)
                ids.append(i)
            pylab.legend(ids, title='IDs', loc='lower left')
    # show the plot
    if None == options.out:
        pylab.show()
    else:
        # check if the file extension is present, if not, add it
        if len(options.out) <= len(options.outtype) or options.outtype != options.out[len(options.out)-len(options.outtype):]:
            pylab.savefig("%s.%s" % (options.out, options.outtype), format=options.outtype)
        else:
            pylab.savefig(options.out, format=options.outtype)
        pylab.close()

if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('--fn', help="an output file from dwgsim_eval", action="append", dest="fns")
    parser.add_option('--id', help="the corresponding ID to add to the plot legened", action="append", dest="ids")
    parser.add_option('--infer-ids', help="infer the ids from the file names", action="store_true", default=True, dest="infer_ids")
    parser.add_option('--xlim', help="the x-axis range", dest="xlim")
    parser.add_option('--ylim', help="the y-axis range", dest="ylim")
    parser.add_option('--out', help="the output file", dest="out")
    parser.add_option('--outtype', help="the output file type (ex. pdf, png)", dest="outtype", default="pdf")
    parser.add_option('--min-mapq', help="minimum mapping quality (inclusive)", dest="min_mapq", type=int, default=0)
    parser.add_option('--no-legend', help="do not add a legend to the plot", dest="no_legend", action="store_true", default=False)
    if len(sys.argv[1:]) < 1:
        parser.print_help()
    else:
        options, args = parser.parse_args()
        main(options)



#!/usr/bin/env python

"""
Runs DWGSIM

usage: dwgsim_wrapper.py [options]
    -e,--eRateFirstread=e: base/color error rate of the first read [0.020]
    -E,--eRateSecondread=E: base/color error rate of the second read [0.020]
    -d,--innerDist=d: inner distance between the two ends [500]
    -s,--stdev=s: standard deviation [50]
    -N,--numReads=N: number of read pairs [1000000]
    -1,--loFirstread=1: length of the first read [70]
    -2,--loSecondread=2: length of the second read [70]
    -r,--mutRate=r: rate of mutations [0.0010]
    -R,--fracIndels=R: fraction of mutations that are indels [0.10]
    -X,--indelExt=X: probability an indel is extended [0.30]
    -y,--randProb=y: probability of a random DNA read [0.10]
    -n,--maxN=n: maximum number of Ns allowed in a given read [0]
    -c,--platformType=c: generate reads for Illumina (nuc space), for SOLiD (in color space) or Ion Torrent 
    -S,--strand=S: strand 0: default, 1: same strand, 2: opposite strand
    -f,--flowOrder=f: the flow order for Ion Torrent data [(null)]
    -O,--splitOutput=O: which output needs to be reported, one: one single; two: two files; all: all output options
    -H,--haploid=H: haploid mode
    -i,--input=i: the reference genome FASTA
    -3,--outputBFAST=3: the BFAST output FASTQ
    -4,--outputBWA1=4: the first BWA output FASTQ
    -5,--outputBWA2=5: the second BWA output FASTQ
    -6,--outputMutations=6: the output mutations TXT
"""

import optparse, os, shutil, subprocess, sys, tempfile

def stop_err( msg ):
    sys.stderr.write( '%s\n' % msg )
    sys.exit()

def run_process ( cmd, name, tmp_dir, buffsize ):
    try:
        tmp = tempfile.NamedTemporaryFile( dir=tmp_dir ).name
        tmp_stderr = open( tmp, 'wb' )
        proc = subprocess.Popen( args=cmd, shell=True, cwd=tmp_dir, stderr=tmp_stderr.fileno() )
        returncode = proc.wait()
        tmp_stderr.close()
        # get stderr, allowing for case where it's very large
        tmp_stderr = open( tmp, 'rb' )
        stderr = ''
        try:
            while True:
                stderr += tmp_stderr.read( buffsize )
                if not stderr or len( stderr ) % buffsize != 0:
                    break
        except OverflowError:
            pass
        tmp_stderr.close()
        if returncode != 0:
            raise Exception, stderr
    except Exception, e:
        raise Exception, 'Error in \'' + name + '\'. \n' + str( e )

def check_output ( output, canBeEmpty ):
    if 0 < os.path.getsize( output ):
        return True
    elif False == canBeEmpty:
        raise Exception, 'The output file is empty:' + output

def __main__():
    #Parse Command Line
    parser = optparse.OptionParser()
    parser.add_option( '-e', '--eRateFirstread', dest='eRateFirstread', type='float', help='base/color error rate of the first read' )
    parser.add_option( '-E', '--eRateSecondread', dest='eRateSecondread', type='float', help='base/color error rate of the second read' )
    parser.add_option( '-d', '--innerDist', dest='innerDist', type='int', help='inner distance between the two ends' )
    parser.add_option( '-s', '--stdev', dest='stdev', type='float', help='standard deviation' )
    parser.add_option( '-N', '--numReads', dest='numReads', type='int', help='number of read pairs' )
    parser.add_option( '-r', '--mutRate', dest='mutRate', type='float', help='rate of mutations' )
    parser.add_option( '-R', '--fracIndels', dest='fracIndels', type='float', help='fraction of mutations that are indels' )
    parser.add_option( '-X', '--indelExt', dest='indelExt', type='float', help='probability an indel is extended' )
    parser.add_option( '-y', '--randProb', dest='randProb', type='float', help='probability of a random DNA read' )
    parser.add_option( '-n', '--maxN', dest='maxN', type='int', help='maximum number of Ns allowed in a given read' )
    parser.add_option( '-c', '--platformType', dest='platformType', type='int', help='Platform to simulate: 0: Illumina; 1: solid; 2: Ion Torrent' )
    parser.add_option( '-S', '--strand', dest='strand', type='choice', default='0', choices=('0', '1', '2'), help='strand 0: default, 1: same strand, 2: opposite strand' )
    parser.add_option( '-f', '--flowOrder', dest='flowOrder', type='int', default='2')
    parser.add_option( '-H', '--haploid', action='store_true', dest='haploid', default=False, help='haploid mode' )
    parser.add_option( '-i', '--input', dest='input', help='The reference genome fasta' )
    parser.add_option( '-O', dest='splitoutput', type='choice', default='two',choices=('two', 'one', 'all'), help='one: one single; two: two files; all: all output options' )
    parser.add_option( '-1', '--loFirstread', dest='loFirstread', type='int', help='length of the first read' )
    parser.add_option( '-2', '--loSecondread', dest='loSecondread', type='int', help='length of the second read' )
    parser.add_option( '-6', '--outputMutations', dest='outputMutations', help='the output mutations TXT' )
    parser.add_option( '-4', '--outputBWA1', dest='outputBWA1', help='the first BWA output FASTQ' )
    parser.add_option( '-5', '--outputBWA2', dest='outputBWA2', help='the second BWA output FASTQ' )
    parser.add_option( '-3', '--outputBFAST', dest='outputBFAST', help='the BFAST output FASTQ' )
							
    (options, args) = parser.parse_args()

    # output version # of tool
    try:
        tmp = tempfile.NamedTemporaryFile().name
        tmp_stdout = open( tmp, 'wb' )
        proc = subprocess.Popen( args='dwgsim 2>&1', shell=True, stdout=tmp_stdout )
        tmp_stdout.close()
        returncode = proc.wait()
        stdout = None
        for line in open( tmp_stdout.name, 'rb' ):
            if line.lower().find( 'version' ) >= 0:
                stdout = line.strip()
                break
        if stdout:
            sys.stdout.write( '%s\n' % stdout )
        else:
            raise Exception
    except:
        sys.stdout.write( 'Could not determine DWGSIM version\n' )

    buffsize = 1048576

    # make temp directory for dwgsim, requires trailing slash
    tmp_dir = '%s/' % tempfile.mkdtemp()

    #'generic' options used in all dwgsim commands here

    try:
        reference_filepath = tempfile.NamedTemporaryFile( dir=tmp_dir, suffix='.fa' ).name
        os.symlink( options.input, reference_filepath )
        assert reference_filepath and os.path.exists( reference_filepath ), 'A valid genome reference was not provided.'
        tmp_dir = '%s/' % tempfile.mkdtemp()
        dwgsim_output_prefix = "dwgsim_output_prefix"
        dwgsim_cmd = 'dwgsim -e %s -E %s -d %s -s %s -N %s -1 %s -2 %s -r %s -R %s -X %s -y %s -c %s -n %s' % \
                (options.eRateFirstread, \
                options.eRateSecondread, \
                options.innerDist, \
                options.stdev, \
                options.numReads, \
                options.loFirstread, \
                options.loSecondread, \
                options.mutRate, \
                options.fracIndels, \
                options.indelExt, \
                options.randProb, \
                options.platformType, \
                options.maxN)
        if options.haploid:
            dwgsim_cmd = dwgsim_cmd + ' -H'
        if options.platformType == "2":
            dwgsim_cmd = dwgsim_cmd + options.flowOrder
        dwgsim_cmd = dwgsim_cmd + ' ' + options.input
        dwgsim_cmd = dwgsim_cmd + ' ' + tmp_dir + dwgsim_output_prefix
        
        # temporary file to check the command being launched
        tmpfile = open('/tmp/dwgsim.log', 'w')
        tmpfile.write("%s" % dwgsim_cmd)

        # need to nest try-except in try-finally to handle 2.4
        try:
			# launch dwgsim + accept mutations.txt output
			run_process ( dwgsim_cmd, 'dwgsim', tmp_dir, buffsize )
			cmd = 'mv ' + tmp_dir + '/' + dwgsim_output_prefix + '.mutations.txt' + ' ' + options.outputMutations
			run_process ( cmd, 'mv #1', tmp_dir, buffsize )
			check_output ( options.outputMutations, True )
			# check which output required and move files accordingly
			tmpfile.write("\nsplitoutput: %s \ncondition one: %s \ncondition two: %s \ncondition all: %s\n" % (options.splitoutput, str((options.splitoutput == 'one')), str((options.splitoutput == 'two')), str((options.splitoutput == 'all'))))
			if  any( [options.splitoutput == 'two' , options.splitoutput == 'all'] ) :	# two files
				cmd = 'mv ' + tmp_dir + '/' + dwgsim_output_prefix + '.bwa.read1.fastq' + ' ' + options.outputBWA1
				run_process ( cmd, 'mv #3', tmp_dir, buffsize )
				cmd = 'mv ' + tmp_dir + '/' + dwgsim_output_prefix + '.bwa.read2.fastq' + ' ' + options.outputBWA2
				run_process ( cmd, 'mv #4', tmp_dir, buffsize )
				tmpfile.write("\nmoved!!: %s" % options.splitoutput)
				# check that there are results in the output file
				check_output ( options.outputBWA1, False )
				check_output ( options.outputBWA2, False )
			if any( [options.splitoutput == 'one' , options.splitoutput == 'all'] ):
				cmd = 'mv ' + tmp_dir + '/' + dwgsim_output_prefix + '.bfast.fastq' + ' ' + options.outputBFAST
				run_process ( cmd, 'mv #2', tmp_dir, buffsize )
				# check that there are results in the output file
				check_output ( options.outputBFAST, False )
			sys.stdout.write( 'DWGSIM successful' )
        except Exception, e:
			stop_err( 'DWGSIM failed.\n' + str( e ) )
    finally:
        # clean up temp dir
        if os.path.exists( tmp_dir ):
            shutil.rmtree( tmp_dir )

if __name__=="__main__": __main__()



#!/usr/bin/env python

"""
Runs DWGSIM_EVAL

usage: dwgsim_eval_wrapper.py [options]
    -a,--alignmentScore=a: split alignments by alignment score instead of mapping quality
    -b,--bwa=b: alignments are from BWA (SOLiD only)
    -c,--colorSpace=c: color space alignments
    -d,--scoreFactor=d: divide quality/alignment score by this factor
    -g,--wiggle=g: gap "wiggle"
    -n,--numReads=n: number of raw input paired-end reads (otherwise, inferred from all SAM records present).
    -q,--minMapq=q: consider only alignments with this mapping quality or greater.
    -z,--singleEnd=z: input contains only single end reads
    -S,--sam=s: input SAM file
    -B,--bam=B: input BAM file
    -p,--printIncorrect=p: print incorrect alignments
    -s,--numSnps=s: consider only alignments with the number of specified SNPs
    -e,--numErrors=e: consider only alignments with the number of specified errors
    -i,--indels=i: consider only alignments with indels
    -o,--output=u: The file to save the output 
"""

import optparse, os, shutil, subprocess, sys, tempfile

def stop_err( msg ):
    sys.stderr.write( '%s\n' % msg )
    sys.exit()

def run_process ( cmd, name, tmp_dir, buffsize ):
    try:
        tmp = tempfile.NamedTemporaryFile( dir=tmp_dir ).name
        tmp_stderr = open( tmp, 'wb' )
        proc = subprocess.Popen( args=cmd, shell=True, cwd=tmp_dir, stderr=tmp_stderr.fileno() )
        returncode = proc.wait()
        tmp_stderr.close()
        # get stderr, allowing for case where it's very large
        tmp_stderr = open( tmp, 'rb' )
        stderr = ''
        try:
            while True:
                stderr += tmp_stderr.read( buffsize )
                if not stderr or len( stderr ) % buffsize != 0:
                    break
        except OverflowError:
            pass
        tmp_stderr.close()
        if returncode != 0:
            raise Exception, stderr
    except Exception, e:
        raise Exception, 'Error in \'' + name + '\'. \n' + str( e )

def check_output ( output, canBeEmpty ):
    if 0 < os.path.getsize( output ):
        return True
    elif False == canBeEmpty:
        raise Exception, 'The output file is empty:' + output

def __main__():
    #Parse Command Line
    parser = optparse.OptionParser()

    parser.add_option( '-a', '--alignmentScore', action='store_true', dest='alignmentScore', default=False, help='split alignments by alignment score instead of mapping quality' )
    parser.add_option( '-b', '--bwa', action='store_true', dest='bwa', default=False, help='alignments are from BWA (SOLiD only)' )
    parser.add_option( '-c', '--colorSpace', action='store_true', dest='colorSpace', default=False, help='generate reads in color space (SOLiD reads)' )
    parser.add_option( '-d', '--scoreFactor', dest='scoreFactor', type='int', help='divide quality/alignment score by this factor' )
    parser.add_option( '-g', '--wiggle', dest='wiggle', type='int', help='gap "wiggle"' )
    parser.add_option( '-n', '--numReads', dest='numReads', type='int', help='number of raw input paired-end reads (otherwise, inferred from all SAM records present).' )
    parser.add_option( '-q', '--minMapq', dest='minMapq', type='int', help='consider only alignments with this mapping quality or greater.' )
    parser.add_option( '-z', '--singleEnd', action='store_true', dest='singleEnd', default=False, help='input contains only single end reads' )
    parser.add_option( '-S', '--sam', dest='sam', default=None, help='input SAM' )
    parser.add_option( '-B', '--bam', dest='bam', default=None, help='input BAM' )
    parser.add_option( '-p', '--printIncorrect', action='store_true', dest='printIncorrect', default=False, help='print incorrect alignments' )
    parser.add_option( '-s', '--numSnps', dest='numSnps', type="int", help='consider only alignments with the number of specified SNPs' )
    parser.add_option( '-e', '--numErrors', dest='numErrors', type="int", default=False, help='consider only alignments with the number of specified errors' )
    parser.add_option( '-i', '--indels', action='store_true', dest='indels', default=False, help='consider only alignments with indels' )
    parser.add_option( '-o', '--output', dest='output', help='The file to save the output' )
    
    (options, args) = parser.parse_args()

    # output version # of tool
    try:
        tmp = tempfile.NamedTemporaryFile().name
        tmp_stdout = open( tmp, 'wb' )
        proc = subprocess.Popen( args='dwgsim_eval 2>&1', shell=True, stdout=tmp_stdout )
        tmp_stdout.close()
        returncode = proc.wait()
        stdout = None
        for line in open( tmp_stdout.name, 'rb' ):
            if line.lower().find( 'version' ) >= 0:
                stdout = line.strip()
                break
        if stdout:
            sys.stdout.write( '%s\n' % stdout )
        else:
            raise Exception
    except:
        sys.stdout.write( 'Could not determine DWGSIM_EVAL version\n' )

    buffsize = 1048576

    # make temp directory for dwgsim, requires trailing slash
    tmp_dir = '%s/' % tempfile.mkdtemp()

    #'generic' options used in all dwgsim commands here

    try:
        tmp_dir = '%s/' % tempfile.mkdtemp()
        dwgsim_eval_cmd = 'dwgsim_eval'
        if True == options.alignmentScore:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -a'
        if True == options.bwa:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -b'
        if True == options.colorSpace:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -c'
        use_p = False
        if 0 <= options.numSnps:
            use_p = True
            dwgsim_eval_cmd = dwgsim_eval_cmd + (' -s %s' % options.numSnps)
        if 0 <= options.numErrors:
            use_p = True
            dwgsim_eval_cmd = dwgsim_eval_cmd + (' -e %s' % options.numErrors)
        if True == options.indels:
            use_p = True
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -i'
        if True == use_p or True == options.printIncorrect:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -p'
        if True == options.singleEnd:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -z'
        dwgsim_eval_cmd = '%s -d %s -g %s -n %s -q %s' % (dwgsim_eval_cmd, \
                options.scoreFactor, \
                options.wiggle, \
                options.numReads, \
                options.minMapq)
        if None != options.sam:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' -S ' + options.sam
        elif None != options.bam:
            dwgsim_eval_cmd = dwgsim_eval_cmd + ' ' + options.bam
        else:
            raise Exception, 'Input file was neither a SAM nor BAM'
        dwgsim_eval_cmd = dwgsim_eval_cmd + ' > ' + options.output

        # need to nest try-except in try-finally to handle 2.4
        try:
            # dwgsim
            run_process ( dwgsim_eval_cmd, 'dwgsim', tmp_dir, buffsize )
            # check that there are results in the output file
            check_output ( options.output, False )
            sys.stdout.write( 'DWGSIM_EVAL successful' )
        except Exception, e:
            stop_err( 'DWGSIM_EVAL failed.\n' + str( e ) )
    finally:
        # clean up temp dir
        if os.path.exists( tmp_dir ):
            shutil.rmtree( tmp_dir )

if __name__=="__main__": __main__()
